// ----- circle.h

#ifndef CIRCLE_H
#define CIRCLE_H
#include <graphics.h>

namespace GraphicsLibrary  {

template <class T>
class Circle : public TShape<T>  {
protected:
    int radius;
	bool sorted;
	std::vector<Point> points;
	virtual void drawpoint(Point pt);
	virtual Point translate(Point pt);
    virtual void drawfilled();
private:
	void draw8points(int x, int y);
	void draw4points(int x, int y);
public:
    Circle(T& t, Point center, int rd, Color clr = black, bool fill = false) : 
			TShape<T>(&t, center, clr, fill), radius(rd), sorted(false)
    {  }
    virtual void draw();
};
template <class T>
Point Circle<T>::translate(Point pt)
{
	sorted = false;
	points.clear();
	return TShape<T>::translate(pt);
}
template <class T>
void Circle<T>::drawpoint(Point pt)
{
	if (filled)
		points.push_back(pt);
	else
		TShape<T>::drawpoint(pt);
}
template <class T>
inline void Circle<T>::draw4points(int x, int y)
{
	drawpoint(Point( x,  y));
	drawpoint(Point(-x,  y));
	drawpoint(Point( x, -y));
	drawpoint(Point(-x, -y));
}
template <class T>
inline void Circle<T>::draw8points(int x, int y)
{
	draw4points(x, y);
	draw4points(y, x);
}
template <class T>
void Circle<T>::draw()
{
	if (radius > 0)	{
		if (!filled || points.size() == 0)	{
			int x = 0, y = radius;
			int d = 1 - radius;
			int d1 = 3, d2 = -2 * radius + 5;
			do	{
				draw8points(x, y);
				if (d < 0)	{
					d += d1;
					d2 += 2;
				}
				else	{
					d += d2;
					d2 += 4;
					--y;
				}
				d1 += 2;
			} while (y > x++);
		}
		if (filled)
			drawfilled();
	}
}

template <class T>
void Circle<T>::drawfilled()
{
	if (!sorted)	{
		std::sort(points.begin(), points.end(), Point::yxcomp);
		sorted = true;
	}
	std::vector<Point>::const_iterator iter = points.begin();
	while (iter < points.end())	{
		int y = iter->y;
		int x1 = iter->x;
		int x2;
		while (iter != points.end() && y == iter->y)
			x2 = (iter++)->x;
		while (x1 <= x2)
			TShape<T>::drawpoint(Point(x1++, y));
	}
}

} // namespace GraphicsLibrary

#endif