// ----- ellipse.h

#ifndef ELLIPSE_H
#define ELLIPSE_H
#include <circle.h>

namespace GraphicsLibrary  {

template <class T>
class Ellipse : public Circle<T>  {
    int yradius;
public:
	Ellipse(T& t, Point center, int xrad, int yrad, Color clr = black, bool fill = false) : 
			Circle<T>(t, center, xrad, clr, fill), yradius(yrad)
    {  }
    virtual void draw();
	virtual void drawpoint(Point pt);
};
template <class T>
void Ellipse<T>::drawpoint(Point pt)
{
	rotate(pt);
	Circle<T>::drawpoint(pt);
}
template <class T>
void Ellipse<T>::draw()
{
	if (!filled || points.size() == 0)	{
		// draw top and bottom points
		drawpoint(Point(0, yradius));
		drawpoint(Point(0, -yradius));

		// draw the four arcs
		Point working(0, yradius);
		long asquared = (long) radius * radius;
		long bsquared = (long) yradius * yradius;
		Point adjust(0, asquared * 2 * yradius);
		long threshold = asquared / 4 - asquared * yradius;

		while (adjust.x < adjust.y)	{
			threshold += adjust.x + bsquared;
			if (threshold >= 0)	{
				adjust.y -= asquared * 2;
				threshold -= adjust.y;
				working.y--;
			}
			adjust.x += bsquared * 2;
			working.x++;
			if (adjust.x < adjust.y)	{
				drawpoint(Point( working.x,-working.y));
				drawpoint(Point(-working.x,-working.y));
				drawpoint(Point( working ));
				drawpoint(Point(-working.x, working.y));
			}
		}

		drawpoint(Point( radius, 0));
		drawpoint(Point(-radius, 0));

		working = Point(radius, 0);
		adjust = Point(bsquared * 2 * radius, 0);
		threshold = bsquared / 4 - bsquared * radius;

		while (adjust.y <= adjust.x)	{
			threshold += adjust.y + asquared;
		
			if (threshold >= 0)	{
				adjust.x -= bsquared * 2;
				threshold -= adjust.x;
				working.x--;
			}
			adjust.y += asquared * 2;
			working.y++;
			if (adjust.y <= adjust.x)	{
				drawpoint(Point( working.x,-working.y));
				drawpoint(Point(-working.x,-working.y));
				drawpoint(Point( working ));
				drawpoint(Point(-working.x, working.y));
			}
		}
	}
	if (filled)
		drawfilled();
}

} // namespace GraphicsLibrary

#endif
