// ------ tshape.h
#ifndef TSHAPE_H
#define TSHAPE_H

#include <vector>
#include <algorithm>
#include <shape.h>

namespace GraphicsLibrary  {

struct Point {
	int x, y;
	Point(int xp = 0, int yp = 0) : x(xp), y(yp) { }
	Point operator+=(const Point& pt);
	Point operator+(const Point& pt) const;
	static bool yxcomp(Point pt1, Point pt2);
};
inline Point Point::operator+=(const Point& pt)
{
	x += pt.x;
	y += pt.y;
	return *this;
}
inline Point Point::operator+(const Point& pt) const
{
	return Point(x + pt.x, y + pt.y);
}
inline bool Point::yxcomp(Point pt1, Point pt2)
{
	return pt1.y == pt2.y ? pt1.x < pt2.x : pt1.y < pt2.y;
}

template <class T>
class TShape : public Shape {
protected:
	Point position;	// position of shape
    T* tp;			// generic graphical rendering engine
	int rotation;	// degrees of rotation

    TShape(T* t, Point pos, Color clr, bool fill = false) : 
			Shape(clr, fill), position(pos), tp(t), rotation(0)
		{  }
	virtual void draw() = 0;
	virtual void drawpoint(Point pt);
	void rotate(Point& pt);
public:
	Point getposition() const
		{ return Point(x, y); }
	virtual Point translate(Point pt);
	virtual void setrotation(int degrees);
};

template <class T>
Point TShape<T>::translate(Point pt)
{
	std::swap(position, pt);
	return pt;
}
template <class T>
void TShape<T>::drawpoint(Point pt)
{
	tp->drawpoint(pt.x + position.x, pt.y + position.y, color);
}
template <class T>
void TShape<T>::setrotation(int degrees)
{
	rotation = degrees;
	while (rotation >= 360)
		rotation -= 360;
	while (rotation <= -360)
		rotation += 360;
}

template <class T>
void TShape<T>::rotate(Point& pt)
{
	if (rotation != 0)	{
	    const double pi = 3.1415927;
        double a = rotation * pi / 180.0;
		Point ptr(
			static_cast<int>(pt.x * std::cos(a) + pt.y * -std::sin(a)),
			static_cast<int>(pt.x * std::sin(a) + pt.y * std::cos(a))
		);
		pt = ptr;
	}
}

} // namespace GraphicsLibrary

#endif
