// ------- line.h

#ifndef LINE_H
#define LINE_H
#include <graphics.h>

namespace GraphicsLibrary  {

template <class T>
class Line : public TShape<T>  {
	Point endpt;
public:
	Line(T& t, Point beg, Point end, Color clr = black) : 
				TShape<T>(&t, beg, clr), endpt(end)
		{  }
	virtual void draw();
};

template <class T>
void Line<T>::draw()
{
	Point beg(0,0);
	Point end(endpt.x - position.x, endpt.y - position.y);
	rotate(beg);
	rotate(end);

	if (beg.x == end.x)  {
        // --- vertical line
		if (end.y < beg.y)
			std::swap(beg, end);
		while (beg.y <= end.y)	{
			drawpoint(beg);
			beg.y++;
		}
    }
	else if (beg.y == end.y)  {
        // ---- horizontal line
		if (end.x < beg.x)
			std::swap(beg, end);
		while (beg.x <= end.x)	{
			drawpoint(beg);
			beg.x++;
		}
    }
	else	{
		// --- force left to right plotting
		if (beg.x > end.x)
			std::swap(beg, end);
		// --- build the first point
		drawpoint(beg);
		int lenx = end.x-beg.x;	// length of the x axes
		int leny = end.y-beg.y;	// length of the y axes
		int inctrack = 1;		// increment the axes tracked
		int incslope = 1;		// increment the slope axes
		int str, len;
		int* slope;
		int* track;
		if (leny < 0)	{
			// -- the y axes goes in reverse
			leny = -leny;
			if (leny < lenx)
				incslope = -1;
			else
				inctrack = -1;
		}
		if (leny < lenx)	{
			// --- track the x axes, slope the y axes
			str = leny * 2;
			len = lenx;
			slope = &beg.y;
			track = &beg.x;
		}
		else	{
			// --- track the y axes, slope the x axes
			str = lenx * 2;
			len = leny;
			slope = &beg.x;
			track = &beg.y;
		}
		int d = str - len;
		int ang = d - len;
		// --- plot the line
		while (len--)	{
			*track += inctrack;
			if (d <= 0)
				d += str;
			else	{
				*slope += incslope;
				d += ang;
			}
			drawpoint(beg);
		}
	}
}


} // namespace GraphicsLibrary

#endif