#ifndef SPECIALIZE_H
#define SPECIALIZE_H

#include <tyfc.h>
#include <vector>
#include <circle.h>
#include <rectangle.h>
#include <line.h>
#include <polygon.h>

namespace GL = GraphicsLibrary;

// --- specialized draw functions for derived TYFCApp class instantiations

void GL::Line<TYFCApp>::draw()
{
	int x1 = tp->adjx(position.x);
	int y1 = tp->adjy(position.y);
	int x2 = tp->adjx(endpt.x);
	int y2 = tp->adjy(endpt.y);
	HDC hdc = tp->Hdc();
    ::MoveToEx(hdc, x1, y1, 0);
    ::LineTo(hdc, x2, y2);
}

void GL::Circle<TYFCApp>::draw()
{
	int x1 = tp->adjx(position.x);
	int y1 = tp->adjy(position.y);
	HDC hdc = tp->Hdc();
	::Ellipse(hdc, x1 - radius, y1 - radius, x1 + radius, y1 + radius);
}

void GL::Ellipse<TYFCApp>::draw()
{
	int x1 = tp->adjx(position.x);
	int y1 = tp->adjy(position.y);
	HDC hdc = tp->Hdc();
	::Ellipse(hdc, x1 - radius, y1 - yradius, x1 + radius, y1 + yradius);
}

void GL::Rectangle<TYFCApp>::draw()
{
	int x1 = tp->adjx(position.x);
	int y1 = tp->adjy(position.y);
	HDC hdc = tp->Hdc();
	::Rectangle(hdc, x1, y1, x1 + width - 1, y1 + height - 1);
}

typedef BOOL (*PolyFunc)(HDC, CONST POINT*, int);

inline void drawpolything(TYFCApp* tp, int x, int y, 
					const std::vector<GL::Point>& vertices, PolyFunc func)
{
	int count = vertices.size();
	x = tp->adjx(x);
	y = tp->adjy(y);
	POINT* ppt = new POINT[count];
	for (int i = 0; i < count; i++)	{
		(ppt+i)->x = vertices[i].x + x;
		(ppt+i)->y = vertices[i].y + y;
	}
	func(tp->Hdc(), ppt, count);
	delete [] ppt;
}

void GL::Polygon<TYFCApp>::draw()
{
	drawpolything(tp, position.x, position.y, vertices, reinterpret_cast<PolyFunc>(&::Polygon));
}

void GL::Polyline<TYFCApp>::draw()
{
	drawpolything(tp, position.x, position.y, vertices, reinterpret_cast<PolyFunc>(&::Polyline));
}

#endif
