/*
This file and all other related Python Server Pages files are Copyright (c) 1999 Angell
Enterprises, Inc.  All rights reserved.
*/


package com.vmc.psp;

import java.util.*;
import java.util.Hashtable;
import javax.servlet.*;
import javax.servlet.http.*;
import org.python.util.PythonInterpreter;
import org.python.core.*;

public class PSP {

	static PythonInterpreter interp = new PythonInterpreter();
	static PyObject codeGenerator = null;
	static Hashtable apps = new Hashtable(); // Cache for Application Contexts
	static String pspRoot = findRoot();

	static final String GLOBAL_SCRIPT = "global.psa";

  // finds the directory where psp is installed
  public static String findRoot() {

      String root;

      // If find psp.jar in class.path
      String classpath = System.getProperty("java.class.path");
      if (classpath == null) return null;

      int jpy = classpath.toLowerCase().indexOf("psp.jar");
      if (jpy == -1) {
          return null;
      }
      int start = classpath.lastIndexOf(java.io.File.pathSeparator, jpy)+1;
      return classpath.substring(start, jpy);

  } // findRoot

  // adds a new application context to the cache
  static public synchronized void addApp( String name, PSPAppContext app ) {
  	apps.put( name, app );
  }
  // gets a cached application context from the cache
	static public synchronized PSPAppContext getApp( String name ) {
  	return (PSPAppContext)apps.get( name );
  }

  // clears the application cache for the servlet
  static public synchronized void clearCache( boolean newValue ) {
  	apps = new Hashtable();
    System.gc();
  }

  // returns a python dictionary with statistics for the loaded apps
  static public synchronized PyDictionary getAppStats() {

  	Hashtable ht = new Hashtable( apps.size() );
    Enumeration e = apps.elements();
    while ( e.hasMoreElements() ) {

    	PSPAppContext app = (PSPAppContext)e.nextElement();
    	Hashtable htPages = new Hashtable( app.g_scripts.size() );
      Enumeration scripts = app.g_scripts.keys();
			Enumeration dates = app.g_dates.elements();

      while ( scripts.hasMoreElements() ) {

      	PyString script = new PyString( (String)scripts.nextElement() );
        PyLong date = new PyLong( ((Long)dates.nextElement()).longValue() );
				htPages.put( script, date );

      } // While

      ht.put( Py.java2py(app.g_appName), new PyDictionary(htPages) );

    } // while

    return new PyDictionary( ht );
    
  } // getAppStats

  // generate a PyDictionary containing HTTP cookies
  static public PyDictionary makeCookies( Cookie[] cookies ) {

  	if ( cookies == null )
    	return new PyDictionary();
      
  	Hashtable ht = new Hashtable( cookies.length );

    for( int i = 0; i < cookies.length; i++ ) {
    	Cookie cookie = cookies[i];

      ht.put( Py.java2py( cookie.getName() ),
      	Py.java2py( cookie ) );

    } // for

    return new PyDictionary( ht );

  } // makeCookies
}
