/*
This file and all other related Python Server Pages files are Copyright (c) 1999 Angell
Enterprises, Inc.  All rights reserved.
*/

package com.vmc.psp;

import java.io.*;
import java.util.Hashtable;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import org.python.util.PythonInterpreter;
import org.python.core.*;

// class PSPAppContext
// Contains the context information maintained for each PSP
// application.
class PSPAppContext {

  static final String APP_START = "Application_OnStart";
  static final String APP_END = "Application_OnEnd";
  static final String SESSION_START = "Session_OnStart";
  static final String SESSION_END = "Session_OnEnd";

	Trace m_tr = null;
  boolean g_globalsLoaded = false;
	String g_appName = null;
  PyObject g_WriteMethod = null;
  PyObject g_FlushResponse = null;
	PythonInterpreter g_interp;
  PyStringMap g_dict = null;
	Hashtable g_scripts = new Hashtable(); // Cache for PyServer Pages
	Hashtable g_dates = new Hashtable();   // Timestamps for Cache entries

  public PSPAppContext( PyStringMap dict, Trace tr, String name ) {

    m_tr = tr;
    g_appName = name;

    g_dict = dict.copy();
    g_interp = new PythonInterpreter( g_dict );

    m_tr.WriteMessage( "App loaded: " + name );
    //exec( "import sys\ndef __write__( s ): Response.write(s)" );
    g_WriteMethod =   __builtin__.compile(
        "import sys\ndef __write__( s ): Response.write(s)",
        "<string>", "exec" );

    g_FlushResponse = __builtin__.compile( "Response.flush()",
      "<string>", "exec" );

  }

  // loadGlobals
  // Loads the 'global.psa' file (if it exists) for this application.
  synchronized void loadGlobals(HttpServletRequest req) {

    if ( g_globalsLoaded )
      return;
      
    g_globalsLoaded = true;

  	String globalScript = req.getRealPath(
    	g_appName + File.separatorChar + PSP.GLOBAL_SCRIPT);

    File f = new File(globalScript);

    if (f.exists())
    	execfile( globalScript );

    applicationStart();

  } // loadGlobals

  // applicationStart
  // Notifies the PSP application that it has started.
  void applicationStart () {

  	if (g_dict.has_key( new PyString(APP_START) )) {
    	exec( APP_START + "()" );
    }

  }

  // applicationEnd
  // Notifies the PSP application that it has ended.
  void applicationEnd () {

  	if (g_dict.has_key( new PyString(APP_END) )) {
    	exec( APP_END + "()" );
    }

  }

  // sessionStart
  // Notifies the PSP application that a session has started.
  synchronized void sessionStart (HttpServletRequest req, HttpServletResponse res) {

   	if (g_dict.has_key( new PyString(SESSION_START) )) {

      ExecContext ec = new ExecContext( req, res, this, false);

	  PyCode pc = __builtin__.compile( 
		SESSION_START + "( Session, Request, Response, Server )", "<string>", "exec" );
	  
	  try {

      	Py.exec( pc, ec.Locals, ec.Locals );
      
	  } catch (PyException e) {

		if ( e.type != Py.SystemExit )
      		throw e;
        
      } finally {

    	ec.release();
      }

    }

  }

  // sessionEnd
  // Notifies the PSP application that a session has ended.
  void sessionEnd (HttpServletRequest req, HttpServletResponse res) {

   	if (g_dict.has_key( new PyString(SESSION_END) )) {
    	exec( SESSION_END + "()", req, res );
    }

  }

  // exec
  // Executes the given python statement(s) within our context
  void exec( PyCode code, HttpServletRequest req, HttpServletResponse res ) {

    ExecContext ec = new ExecContext( req, res, this );
    try {

      Py.exec( code, ec.Locals, ec.Locals );
      
    } catch (PyException e) {

		if ( e.type != Py.SystemExit )
      		throw e;
        
    } finally {

    	ec.release();
    }

  } // exec

  void exec( String script, HttpServletRequest req, HttpServletResponse res ) {

  	exec(
  		__builtin__.compile( script, "<string>", "exec" ),
      req,
      res );

  } // exec

  void exec( String script ) {

  	exec(
  		__builtin__.compile( script, "<string>", "exec" ),
      null,
      null );

  } // exec

  // execfile
  // Executes the given python file within our context
  void execfile( String script, HttpServletRequest req, HttpServletResponse res ) {

    ExecContext ec = new ExecContext( req, res, this );

    try {

    	g_interp.execfile(script);

    } finally {

     	ec.release();
    }

  } // execfile

  void execfile( String script ) {

		execfile( script, null, null );

  } // execfile

  // processPage
  // Executes the given page inside of this application's context.
//  synchronized public void processPage( String psp,
  public void processPage( String psp,
  	HttpServletRequest req, HttpServletResponse res ) {

    if ( !g_globalsLoaded )
    	loadGlobals(req);

    PyCode pyScript = getPythonScript( psp,req );

    exec( pyScript, req, res );

  } // processPage

  // getPythonScript
  // Returns a python file based on the Python Server Page script
  // file.
	synchronized PyCode getPythonScript(String path,HttpServletRequest req) {

    Long date = (Long) g_dates.get(path);
    if (date != null) {                     // Check for cache validity

    	String realPath = req.getRealPath( path );
    	File file = new File(realPath);

      if (date.longValue() < file.lastModified()) {  // Cache invalid

      	// Remove the cached code if it exists
				if (g_scripts.containsKey( path )) {
					m_tr.WriteMessage( "Cached page invalid." );
        } // if file cached

        return ( loadPythonServerPage(path,req) );  // Reload servlet

    	} else

      	return ((PyCode) g_scripts.get(path));

    } else {

      return (loadPythonServerPage(path,req));        // Not in cache, load servlet

    } // if date

  } // getPythonScript

  // loadPythonServerPage
  // Loads the given PSP and runs it through the Code Generator
  // engine.  The results of the engine are stored in a temporary
  // file and the file name returned.
  PyCode loadPythonServerPage(String path, HttpServletRequest req) {

  	String realPath = req.getRealPath( path );
    m_tr.WriteMessage( "PHY " + realPath );

  	File file = new File(realPath);

    PyObject pyTempName = PSP.codeGenerator.invoke( "generate",
      Py.java2py(realPath), new PyInteger(1) );

	String tempName = pyTempName.toString();

    m_tr.WriteMessage( "CREATED " + tempName );

    PyCode code = compile( tempName, path );
	
    if ( code != null ) {

    	m_tr.WriteMessage( "COMPILED " + tempName );
	    g_scripts.put(path, code );        // Add page to cache
  	  	g_dates.put(path, new Long(file.lastModified()));   // Add timestamp

    } else {

		m_tr.WriteMessage( "ERR request could not be compiled." );

	} // else

    return code;

  } // loadPythonServerPage

  // compile
  // Reads a file and sends it through the python compiler
  PyCode compile( String fileName, String pageName ) {

  	StringBuffer contents = null;
  	try {
	 		BufferedReader in
  	 		= new BufferedReader(new FileReader(fileName));

    	contents = new StringBuffer();
	    String line = null;
  	  do {
    		line = in.readLine();
    	  if ( line != null )
      		contents.append( line + "\n" );
	    } while ( line != null );

  	  in.close();
    } catch (Exception e) {

    	return null;
    }

  	PyCode code =
    	__builtin__.compile(contents.toString(), pageName, "exec");

    return code;

  } // compile

  // class ExecContext
	// Initializes a Python interpreter environment and reverses the
	// initialization.
	final class ExecContext {

    public PyObject Locals = g_dict.copy();
    HttpServletRequest m_req = null;
    HttpServletResponse m_res = null;
    PySession m_session = null;
		PSPAppContext m_app = null;

    public ExecContext() {
    	this( (HttpServletRequest)null, (HttpServletResponse)null,
      	null, false );
    }

	public ExecContext( HttpServletRequest req,	HttpServletResponse res,
    	PSPAppContext app ) {

		this( req,res,app,true );
	}

	public ExecContext( HttpServletRequest req,	HttpServletResponse res,
    	PSPAppContext app, boolean startSession ) {

    	m_req = req;
		m_res = res;
 		m_app = app;

      if ( req != null ) {

        m_session = new PySession(req.getSession(true));
        if ( startSession && m_session.m_ses.isNew() ) {
        	sessionStart( req, res );
        }
      }

      if ( req != null ) {
          initDictionary( Locals, req, res );
          Py.exec( g_WriteMethod, Locals, Locals );

      } // If

  	} // Constructor

    public void initDictionary( PyObject dict,HttpServletRequest req,
      HttpServletResponse res ) {

      dict.__setitem__( "Session", m_session );
      dict.__setitem__( "Request", new PyJavaInstance( new PyRequest(req)));
      dict.__setitem__( "Response", new PyJavaInstance( new PyResponse(res) ));
      dict.__setitem__( "Server", new PyJavaInstance(new PyServer( req, m_app )));

    } // initDictionary

  	final void release() {

    	try {

        Py.exec( g_FlushResponse, Locals, Locals );
        Locals = null;

      } catch (Exception e) {}

  	}

	} // ExecContext
}

