
/*
This file and all other related Python Server Pages files are Copyright (c) 1999 Angell
Enterprises, Inc.  All rights reserved.
*/

package com.vmc.psp;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import org.python.core.*;

// PSPServlet
// Java Servlet that processes files with embedded Python
// script.  The Python class com.vmc.gcEngine is used to process
// the file.  The file is then executed and the results written
// to HTTP response.
public class PSPServlet extends HttpServlet {

  Trace tr = new Trace( "PSPServlet.log" );

	PyObject m_cgEngine = null;

  static {

    Trace tr = new Trace( "PSPServlet.log" );

    try {

    	tr.setID( "STARTUP" );
	  tr.WriteMessage( "Python Server Pages " + Version.getVersion() );
	  tr.WriteMessage( Version.getCopyright() );

  	  tr.WriteMessage( "Initializing subsystems..." );

	  // Initialize the JPython interpreter     
	  PSP.interp.exec( "import sys" );
      tr.WriteMessage( "System services available." );
	  tr.WriteMessage( "JPython " + PSP.interp.eval( "sys.version" ).toString() );

	  // Put our installation directory at the beginning of the JPython 
	  // search path.  That means our modules get loaded before anything else,
	  // keeping us away from any nasty module colisions.
      tr.WriteMessage( "Modifying module search path: " + PSP.pspRoot );
      PSP.interp.set( "PSPSearchPath", Py.java2py(PSP.pspRoot) );
      PSP.interp.exec( "sys.path = [PSPSearchPath] + sys.path" );

	  // Load up the JPython based code generator module used by PSP
      tr.WriteMessage( "Loading code generator." );
	  PyObject cg = __builtin__.__import__( new PyString("cg") );
	  PyObject cgVersion = cg.__getattr__( "Version" );
	  tr.WriteMessage( "Python Code Generator " + cgVersion.toString() );

	  // Create an instance of the code generator that we can use
      String cachedir = PSP.pspRoot + "cache";
	  tr.WriteMessage( "Cache directory: " + cachedir );
	  PyObject engineClass = cg.__getattr__( "cgEngine" );
	  PyObject engine = engineClass.__call__( Py.java2py(cachedir) );
	  PSP.codeGenerator = engine;

      tr.WriteMessage( "Initialization complete." );

      tr.WriteMessage( "Ready." );

    } catch (Exception e) {
      tr.WriteException( e, "Unable to initialize PSPServlet." );
    } // catch
  }

	public void service(ServletRequest req, ServletResponse res)
  	throws ServletException, IOException {

    String psp = ((HttpServletRequest)req).getServletPath();

    tr.setID( req.getRemoteHost() );
    tr.WriteMessage( "REQ " + psp );

    // Get an application object to satisify this request.
    PSPAppContext ac = getApplication( (HttpServletRequest)req );
    ac.processPage(
    	psp, (HttpServletRequest)req,
      (HttpServletResponse)res );

    tr.WriteMessage( "REQ Done" );

  } // service

  public String getServletInfo() {
		return "Python Server Pages Engine.";
  } // getServletInfo

  synchronized PSPAppContext getApplication( HttpServletRequest req ) {

  	String psp = req.getServletPath();
    psp = psp.replace( '/', File.separatorChar );
		psp = psp.replace( '\\', File.separatorChar );

  	// Get application name
    File pspFile = new File( psp );
    String appName = pspFile.getParent();

    PSPAppContext app = (PSPAppContext) PSP.getApp( appName );

    if ( app == null )
    	return loadApplication( appName, req );
    else
    	return app;

  } // getApplication

  PSPAppContext loadApplication( String name, HttpServletRequest req ) {

  	File pspFile = new File( name );
    String appName = name;

    // Look for a global script file indicating the
    // base directory of an application.
    while ( appName != null ) {

      	String globalScript = req.getRealPath(
        	appName + File.separatorChar + PSP.GLOBAL_SCRIPT);
    		File f = new File(globalScript);

        if ( f.exists() )
        	break;

        pspFile = new File( appName);
        appName = pspFile.getParent();

    } // while

    if ( appName == null )
    	appName = new String( "" + File.separatorChar);

    PSPAppContext app = (PSPAppContext) PSP.getApp( appName );
    if ( app == null ) {

      app = new PSPAppContext(
    	(PyStringMap)PSP.interp.getLocals(), tr, appName );

      PSP.addApp( appName, app );

    }

    // If we didn't find this application in the same
    // directory where we started
    if ( !appName.equals(name) ) {
    	// Put this application object in the cache
      // under the directory name where we finally
      // found it.
      PSP.addApp( name, app );
    }

    return app;

  } // loadApplication

}


