/*
This file and all other related Python Server Pages files are Copyright (c) 1999 Angell
Enterprises, Inc.  All rights reserved.
*/

package com.vmc.psp;

import javax.servlet.*;
import javax.servlet.http.*;
import java.lang.Integer;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Date;

import java.util.Calendar;

import org.python.core.*;

public class PyResponse extends PyObject {

	public static PyObject ResponseError;
	public static PyException ResponseError(String message) {
	    return new PyException(PyResponse.ResponseError, message);
	}

	HttpServletResponse m_res;
  ServletOutputStream m_out;

  // Control headers written to the stream before
  // any output
	boolean m_headersWritten = false;
	PyDictionary m_Cookies = null;
  String m_charSet = null;
  String m_contentType = "text/html";
  int m_expires = 0;
	String m_expiresAbsolute = "";
	int m_statusCode = 200;
  String m_statusDescription = "Success";

  // Control output buffering to the response stream.
	boolean m_buffered = false;
  Vector m_buffer = null;

	public PyResponse( HttpServletResponse res ) {
		m_res = res;

    try {
    	m_out = res.getOutputStream();
    } catch (Exception e) {}
	}

  //
  // Methods
  //

  public PyObject encodeUrl( String url ) {
    return Py.java2py( m_res.encodeUrl(TranslateForHTTP(url)) );
  }
  
  public PyDictionary getCookies() {

  	if ( m_Cookies == null )
    	m_Cookies = PSP.makeCookies( null );

    return m_Cookies;

  } // getCookies

  public void redirect( String url )
  	throws java.io.IOException {

    //addHeader( "Location", m_res.encodeRedirectUrl( url ) );
    //m_statusCode = 302;
    //m_statusDescription = "Moved";
  	m_res.sendRedirect( m_res.encodeRedirectUrl( TranslateForHTTP(url) ) );
    // Once this method is called, stop processing the
    // current script
    throw new PyException( Py.SystemExit, "redirect" );
  }

  public void clear() {

  	if ( m_buffered ) {
    	if ( m_buffer != null )
      	m_buffer = new Vector();
    } else
    	throw PyResponse.ResponseError( "Output not buffered." );
  }

	public void addHeader( String name, String value ) {

  	m_res.setHeader( name, value );

  }

  public void flush()
  	throws java.io.IOException {

    if ( !m_headersWritten )
    	writeHeaders();
      
  	if ( m_buffer != null ) {
    	Enumeration e = m_buffer.elements();
      while ( e.hasMoreElements() )
      	m_out.println( (String)e.nextElement() );
    } // If buffered

    m_out.flush();
  }

  void writeHeaders()
  	throws java.io.IOException {

  	if ( !m_headersWritten ) {

    	writeCookies();
      if ( m_charSet != null )
  			m_res.setContentType(
        	m_contentType + "; charset=" + m_charSet );
			else
      	m_res.setContentType( m_contentType );

			if ( m_expiresAbsolute.length() > 0 )
      	m_res.setHeader( "Expires", m_expiresAbsolute );
      else
      	m_res.setHeader( "Expires", MinutesToGMT( m_expires ) );

			if ( m_statusCode >= 400 && m_statusCode <= 599 )
				m_res.sendError( m_statusCode, m_statusDescription );
			else
      	m_res.setStatus( m_statusCode, m_statusDescription );

    } // if

    m_headersWritten = true;

  } // writeHeaders

  String MinutesToGMT( int minutes ) {

		Calendar c = Calendar.getInstance();
    c.add( Calendar.MINUTE, minutes );

    return c.getTime().toGMTString();

  } // MinutesToGMT

  void writeCookies() {

  	if ( m_Cookies != null ) {

    	Class classCookie = null;

      try {
      	classCookie = Class.forName("javax.servlet.http.Cookie");
      } catch (Exception e) {}

			PyList cookies = m_Cookies.values();
      for( int i = 0; i < cookies.__len__(); i++ ) {

      	PyObject po = (PyObject) cookies.__finditem__(i);
      	m_res.addCookie( (Cookie)po.__tojava__(classCookie) );

      } // for

    } // if

  } // writeCookies

  public void write( String s )
  	throws java.io.IOException {

  	if ( !m_headersWritten )
    	writeHeaders();

    	if ( m_buffered ) {
      	if ( m_buffer == null )
        	m_buffer = new Vector();

        m_buffer.addElement( s );

      } else {

				m_out.println( s );

      } // If buffered
  }

  public void binaryWrite( String s )
  	throws java.io.IOException {

    if ( !m_headersWritten )
    	writeHeaders();

    m_out.write( s.getBytes() );
  }

  public PyObject Cookie( String name, String value ) {
  	return Py.java2py(new Cookie( name, value ));
  }
  public PyObject Cookie( String name ) {
  	return Py.java2py(new Cookie( name, "" ));
  }

  public boolean getBuffer() {
  	return m_buffered;
  }
  public void setBuffer( boolean b ) {
  	m_buffered = b;
  }
  public String getContenttype() {
  	return m_contentType;
  }
  public void setContenttype( String s) {
		m_contentType = s;
  }
  public String getCharset() {
  	return m_charSet;
  }
  public void setCharset( String s ) {
  	m_charSet = s;
  }
  public int getExpires() {
  	return m_expires;
  }
  public void setExpires( int i ) {
  	m_expires = i;
  }
  public PyTuple getStatus() {
  	PyObject[] poa = new PyObject[] {
    	new PyInteger( m_statusCode ), Py.java2py(m_statusDescription) };
  	return new PyTuple( poa );
  }
  public void setStatus( PyTuple t )
  	throws java.io.IOException {

  	m_statusCode = t.__finditem__(0).__int__().getValue();
  	m_statusDescription = t.__finditem__(1).toString();
  }

  private String TranslateForHTTP( String source ) {

    char c;
    StringBuffer t = new StringBuffer();
    for( int n = 0; n < source.length(); n++ )
    {
      c = source.charAt(n);

      if ( (c >= 0x00) && (c <= 0x1F) ) {
        t.append( "%" + Integer.toString(c, 16) );
      } else {

        switch (c) {
         case ' ': t.append( "%20" ); break;
         case '"': t.append( "%22" ); break;
         case '<': t.append( "%3C" ); break;
         case '>': t.append( "%3E" ); break;
         case '[': t.append( "%5B" ); break;
         case '\\': t.append( "%5C" ); break;
         case ']': t.append( "%5D" ); break;
         case '^': t.append( "%5E" ); break;
         case '`': t.append( "%60" ); break;
         case '{': t.append( "%7B" ); break;
         case '|': t.append( "%7C" ); break;
         case '}': t.append( "%7D" ); break;
         case '~': t.append( "%7E" ); break;
         case 0x7F: t.append( "%7F" ); break;

         default:
            t.append( c );
            break;
            
        } // switch

      } // if-else
      
    } // for

    return t.toString();

  } // TranslateForHTTP

} // PyResponse

