/*
This file and all other related Python Server Pages files are Copyright (c) 1999 Angell
Enterprises, Inc.  All rights reserved.
*/


package com.vmc.psp;

import java.io.*;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.lang.System;

class Trace
{
	// Constant used to specify that the trace file can grow
	// to fill the available drive space.
	public final static long trcUnlimited = -1;

	// Maximum size the trace file is allowed to grow.
	private long m_nMaxFileSize = 1000000;
	// Name of trace file
	private String m_sFilename = "";
	// TRUE if the file is larger than the m_nMaxFileSize
	private boolean m_bFileTooBig = false;
	// Generic text prefixed to each log entry
	private String m_sID = "";
  // Date Format
  private SimpleDateFormat m_sdf = new SimpleDateFormat( "yyyyMMdd HH:mm:ss.SS" );
  
	// ---------------------------------------------------
	// Constructors
	public Trace() {
	}
	public Trace(final String Filename) {
		OpenFile( Filename );
	}
	
	
	// ---------------------------------------------------
	// Properties
	
	public void setMaxFileSize(final long NewSize)	{
		m_nMaxFileSize = NewSize;
	}
	public long getMaxFileSize() {
		return m_nMaxFileSize;
	}
	
	public void setFilename(final String NewName) {
		m_sFilename = NewName;
	}
	public String getFilename() {
		return m_sFilename;
	}
	
	public boolean FileLimitReached() {
		return m_bFileTooBig;
	}
	
	public void setID(final String NewID) {
		m_sID = NewID;
	}
	public String getID() {
		return m_sID;
	}
	
	// ---------------------------------------------------
	// Methods
	
	// OpenFile
	// Sets the name of the trace file to open.  Does not actually
	// open the file until a message is being written to the trace file.
	public void OpenFile(final String NewFile) {

		setFilename( NewFile );
		
	}
	
	// WriteMessage
	// Writes a string to the trace file.
	public void WriteMessage(final String Message) {
		
		// If we do not have a valid file name
		if ( m_sFilename.length() == 0 )
			return;
		
		// If the log file size is limited
		if ( m_nMaxFileSize != trcUnlimited ) {

      File f = new File(m_sFilename);
      if ( f.exists() )
          if ( f.length() >= m_nMaxFileSize ) {
            m_bFileTooBig = true;
            return;
          } // if
    } // if

    try {

      // Open the file
      PrintWriter out = new PrintWriter(
          new FileWriter( m_sFilename, true ), true );

      String sNewMessage =
        m_sdf.format( new Date() ) + " [" + m_sID + "] " + Message;

      out.println( sNewMessage );

      out.close();

    } catch ( Exception e ) {

      e.printStackTrace( System.err );

    } // catch

	}

	// WriteException
	// Writes an error to the trace file
	public void WriteException( Exception e, final String Description ) {

    WriteMessage( "**** RUNTIME ERROR ****" );
    WriteMessage( e.getMessage() );

    try {

      // Open the file
      PrintWriter out = new PrintWriter(
         new FileWriter( m_sFilename, true ), true );

      e.printStackTrace( out );

      out.close();

    } catch ( Exception ex ) {

      e.printStackTrace( System.err );

    } // catch
    
    WriteMessage ( "**** END RUNTIME ERROR ****" );

	}

	// WriteError
	// Writes a error to the trace file
	public void WriteError( long errno, final String Source,
    final String Description ) {

    WriteMessage( "ERR (" + Long.toString(errno) + ") " +
      Source + " - " + Description );
	}

	// WriteWarning
	// Writes a warning to the trace file
	public void WriteWarning( long errno, final String Source,
    final String Description ) {

    WriteMessage( "WRN (" + Long.toString(errno) + ") " +
        Source + " - " + Description );
	}

}
